@@ -4,44 +4,55 @@ from typing import (
     Any, BinaryIO, Generator, Generic, Iterable, List, Optional, TypeVar, cast
 )
 
+from pyais.exceptions import InvalidNMEAMessageException
 from pyais.messages import NMEAMessage
 from pyais.util import FixedSizeDict
 
-
 F = TypeVar("F", BinaryIO, socket, None)
+DOLLAR_SIGN = ord("$")
+EXCLAMATION_POINT = ord("!")
 
 
-class Stream(Generic[F]):
+def should_parse(byte_str: bytes) -> bool:
+    """Return True if a given byte string seems to be NMEA message.
+    This method does **NOT** validate the message, but uses a heuristic
+    approach to check (or guess) if byte string is a valid nmea_message.
+    """
+    # The byte sequence is not empty and starts with a $ or a ! and has 6 ','
+    return len(byte_str) > 0 and byte_str[0] in (DOLLAR_SIGN, EXCLAMATION_POINT) and byte_str.count(b",") == 6
 
-    def __init__(self, fobj: F) -> None:
-        self._fobj: F = fobj
 
-    def __enter__(self) -> "Stream[F]":
+class AssembleMessages(ABC):
+    """
+    Base class that assembles multiline messages.
+    Offers a iterator like interface.
+
+    This class comes without a __init__ method, because it should never be instantiated!
+    """
+
+    def __enter__(self) -> "AssembleMessages":
         # Enables use of with statement
         return self
 
     def __exit__(self, exc_type: object, exc_val: object, exc_tb: object) -> None:
-        if self._fobj is not None:
-            self._fobj.close()
+        return None
 
     def __iter__(self) -> Generator[NMEAMessage, None, None]:
         return self._assemble_messages()
 
     def __next__(self) -> NMEAMessage:
+        """Returns the next decoded NMEA message."""
         return next(iter(self))
 
     def _assemble_messages(self) -> Generator[NMEAMessage, None, None]:
         queue: List[NMEAMessage] = []
 
         for line in self._iter_messages():
-            # Try to parse the message
-            try:
-                msg: NMEAMessage = NMEAMessage(line)
-            except Exception as e:
-                raise ValueError(f'Failed to parse line "{line!r}"') from e
 
             # Be gentle and just skip invalid messages
-            if not msg.is_valid:
+            try:
+                msg: NMEAMessage = NMEAMessage(line)
+            except InvalidNMEAMessageException:
                 continue
 
             if msg.is_single:
@@ -51,21 +62,82 @@ class Stream(Generic[F]):
             elif msg.is_multi:
                 queue.append(msg)
 
-                if msg.index == msg.count:
+                if msg.fragment_number == msg.message_fragments:
                     yield msg.assemble_from_iterable(queue)
                     queue.clear()
             else:
                 raise ValueError("Messages are out of order!")
 
+    @abstractmethod
     def _iter_messages(self) -> Generator[bytes, None, None]:
-        yield from self.read()
+        raise NotImplementedError("Implement me!")
+
+
+class IterMessages(AssembleMessages):
+
+    def __init__(self, messages: Iterable[bytes]):
+        # If the user passes a single byte string make it into a list
+        if isinstance(messages, bytes):
+            messages = [messages, ]
+        self.messages: Iterable[bytes] = messages
+
+    @classmethod
+    def from_strings(cls, messages: Iterable[str], ignore_encoding_errors: bool = False,
+                     encoding: str = "utf-8") -> "IterMessages":
+        # If the users passes a single message as string, make it a list
+        if isinstance(messages, str):
+            messages = [messages, ]
+
+        encoded: List[bytes] = []
+        for message in messages:
+            try:
+                encoded.append(message.encode(encoding))
+            except UnicodeEncodeError as e:
+                if ignore_encoding_errors:
+                    # Just skip and carry on
+                    continue
+                raise e
+
+        return IterMessages(encoded)
+
+    def _iter_messages(self) -> Generator[bytes, None, None]:
+        # Transform self.messages into a generator
+        yield from (message for message in self.messages)
+
+
+class Stream(AssembleMessages, Generic[F], ABC):
+
+    def __init__(self, fobj: F) -> None:
+        """
+        Create a new Stream-like object.
+        @param fobj: A file-like or socket object.
+        """
+        self._fobj: F = fobj
+
+    def __exit__(self, exc_type: object, exc_val: object, exc_tb: object) -> None:
+        if self._fobj is not None:
+            self._fobj.close()
+
+    def _iter_messages(self) -> Generator[bytes, None, None]:
+        # Do not parse lines, that are obviously not NMEA messages
+        yield from (line for line in self.read() if should_parse(line))
 
     @abstractmethod
     def read(self) -> Generator[bytes, None, None]:
         raise NotImplementedError()
 
 
-class FileReaderStream(Stream[BinaryIO]):
+class BinaryIOStream(Stream[BinaryIO]):
+    """Read messages from a file-like object"""
+
+    def __init__(self, file: BinaryIO) -> None:
+        super().__init__(file)
+
+    def read(self) -> Generator[bytes, None, None]:
+        yield from self._fobj.readlines()
+
+
+class FileReaderStream(BinaryIOStream):
     """
     Read NMEA messages from file
     """
@@ -75,14 +147,12 @@ class FileReaderStream(Stream[BinaryIO]):
         self.mode: str = mode
         # Try to open file
         try:
-            file = cast(BinaryIO, open(self.filename, mode=self.mode))
+            file = open(self.filename, mode=self.mode)
+            file = cast(BinaryIO, file)
         except Exception as e:
             raise FileNotFoundError(f"Could not open file {self.filename}") from e
         super().__init__(file)
 
-    def read(self) -> Generator[bytes, None, None]:
-        yield from self._fobj.readlines()
-
 
 class ByteStream(Stream[None]):
     """
